%% This code was used to perform OptKnock analysis in the article: 
% "Analysis of the propionate metabolism in Bacillus subtilis during 3-indolacetic production"
%  Based on (Burgard et al., 2003).

changeCobraSolver('gurobi','all');
%fullPath = which('tutorial_optKnock');
%folder = fileparts(fullPath);
%currectDirectory = pwd;
%cd(folder);


modelFileName = 'ibsu1.mat';
model= xls2model('ibsu1.xlsx');
modelDirectory = getDistributedModelFolder(modelFileName); %Look up the folder for the distributed Models.
modelFileName= [modelDirectory filesep modelFileName]; % Get the full path. Necessary to be sure, that the right model is loaded
model = readCbModel(modelFileName);

biomass = 'R_bio00006';


%% 
% Define the maximum number of solutions to find (i.e., maximum number of 
% remvable reactions that lead to the overproduction of the metabolite of interest)

threshold = 5;

% Define the set of reactions that will be used to search knockouts. Note 
% : only reactions in this set will be deleted

             
               
selectedRxnList = {'R_GLCabc_ts'; 'R_GLCpts_ts'; 'R_GK'; 'R_PGI'; 'R_PFK'; 'R_FBA'; 'R_TPI'; 'R_GAPD'; ...
                   'R_PGK'; 'R_PGM'; 'R_ENO'; 'R_PYK'; 'R_LDH'; 'R_ALCD'; 'R_PAT'; 'R_ACK'; ...
                   'R_PGL'};


%% 
% Constraint the model with biological assumptions

% Simulaciones para Glucosa/ Trp
model = changeRxnBounds(model,'R_Ex_Glc_e', -1.2, 'b');
model = changeRxnBounds(model,'R_Ex_TRP_e',-1.5,'b');
model =changeRxnBounds(model,'R_Ex_Chorismate_e',0,'b');
model =changeRxnBounds(model,'R_Ex_TYR_e',0,'b');

% Simulaciones con Propionato
model = changeRxnBounds(model,'R_Ex_Propionate_e',-2.85,'b');
model = changeRxnBounds(model,'R_Ex_TRP_e',-1.8,'b');
model =changeRxnBounds(model,'R_Ex_Chorismate_e',0,'b');
model =changeRxnBounds(model,'R_Ex_TYR_e',0,'b');


%%
% Then, calculates the production of metabolites before running optKnock.


fbaWT = optimizeCbModel(model);
aiaFluxWT = fbaWT.x(strcmp(model.rxns, 'AIA_extracellular'));
growthRateWT = fbaWT.f;
fprintf('The production of AIA before optimization is %.1f \n', aiaFluxWT);
fprintf('The growth rate before optimization is %.1f \n', growthRateWT);
%% 
%________________________________________________________________


% * EXAMPLE 1 : AIA OVERPRODUCTION*
% *Aim:* *finding optKnock reactions sets of size 2 for increasing production 
% of IAA*

fprintf('\n...EXAMPLE 1: Finding optKnock sets of size 2 or less...\n\n')
% Set optKnock options
% The exchange of AIA will be the objective of the outer problem
options = struct('targetRxn', 'AIA_extracellular', 'numDel', 2)
% We will impose that biomass be at least 50% of the biomass of wild-type

constrOpt = struct('rxnList', {{biomass}},'values', 0.5*fbaWT.f, 'sense', 'G')
% We will try to find 10 optKnock sets of a maximun length of 2
previousSolutions = cell(10, 1)
contPreviousSolutions = 1
%
threshold = 5;
nIter = 1;

while nIter < threshold
    fprintf('...Performing optKnock analysis...\n')
    nIter
    if isempty(previousSolutions{1})
        optKnockSol = OptKnock(model, selectedRxnList, options, constrOpt);
    else
        optKnockSol = OptKnock(model, selectedRxnList, options, constrOpt, previousSolutions, 1);
    end
    
    % determine IAA production and growth rate after optimization
    aiaFluxM1 = optKnockSol.fluxes(strcmp(model.rxns, 'AIA_extracellular'))
    growthRateM1 = optKnockSol.fluxes(strcmp(model.rxns, biomass));
    setM1 = optKnockSol.rxnList;
    
    if ~isempty(setM1)
        previousSolutions{contPreviousSolutions} = setM1;
        contPreviousSolutions = contPreviousSolutions + 1;
        %printing results
        fprintf('optKnock found a optKnock set of large %d composed by ', length(setM1));
        for j = 1:length(setM1)
            if j == 1
                fprintf('%s', setM1{j});
            elseif j == length(setM1)
                fprintf(' and %s', setM1{j});
            else
                fprintf(', %s', setM1{j});
            end
        end
        fprintf('\n');
        fprintf('The production of AIA after optimization is %.2f \n', aiaFluxM1);
        fprintf('The growth rate after optimization is %.2f \n', growthRateM1);
        %fprintf(['The production of other products such as ethanol, formate, lactate and acetate are' ...
          %       '%.1f, %.1f, %.1f and %.1f, respectively. \n'], etohFluxM1, formFluxM1, lactFluxM1, acetFluxM1);
        fprintf('...Performing coupling analysis...\n');
        [type, maxGrowth, maxProd, minProd] = analyzeOptKnock(model, setM1, 'AIA_extracellular');
        fprintf('The solution is of type: %s\n', type);
        fprintf('The maximun growth rate given the optKnock set is %.2f\n', maxGrowth);
        fprintf(['The maximun and minimun production of succinate given the optKnock set is ' ...
                 '%.2f and %.2f, respectively \n\n'], minProd, maxProd);
        if strcmp(type, 'growth coupled')
            singleProductionEnvelope(model, setM1, 'AIA_extracellular', biomass, 'savePlot', 1, 'showPlot', 1, ...
                                     'fileName', ['aia_ex1_' num2str(nIter)], 'outputFolder', 'OptKnockResults');
        end,
    else
        if nIter == 1
            fprintf('optKnock was not able to found an optKnock set\n');
        else
            fprintf('optKnock was not able to found additional optKnock sets\n');
        end
        break;
    end
    nIter = nIter + 1;
end